@@ -60,6 +60,11 @@ glmfit <- function(y, x, start = NULL, weights = NULL, offset = NULL, cluster =
   }
   args$control <- do.call("glm.control", ctrl)
   
+  ## add intercept-only regressor matrix (if missing)
+  ## NOTE: does not have terms/formula
+  if(is.null(x)) x <- matrix(1, nrow = NROW(y), ncol = 1L,
+    dimnames = list(NULL, "(Intercept)"))
+  
   ## call glm fitting function
   args <- c(list(x = x, y = y, start = start, weights = weights, offset = offset), args)
   z <- do.call("glm.fit", args)
